package dev.jansel.aglaea.utils

import com.github.philippheuer.credentialmanager.domain.OAuth2Credential
import com.github.twitch4j.TwitchClientBuilder
import com.github.twitch4j.chat.events.channel.ChannelMessageEvent
import com.github.twitch4j.eventsub.events.ChannelPointsCustomRewardRedemptionEvent
import dev.jansel.aglaea.botRef
import dev.jansel.aglaea.database.collections.ConfigCollection
import dev.jansel.aglaea.database.collections.ReplayCollection
import dev.jansel.aglaea.logger
import dev.jansel.aglaea.twitchClient
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.behavior.getChannelOf
import dev.kord.core.entity.channel.GuildMessageChannel
import dev.kordex.core.koin.KordExKoinComponent
import io.ktor.client.request.forms.*
import io.ktor.utils.io.*
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking

class Twitch : KordExKoinComponent {
	suspend fun init() {
		twitchClient = TwitchClientBuilder.builder()
			.withEnableHelix(true)
			.withEnableChat(true)
			.withEnableGraphQL(true)
			.withEnableEventSocket(true)
			.withChatAccount(OAuth2Credential("twitch", twitchToken))
			.withClientId(twitchcid)
			.withClientSecret(twitchcs)
			.build()

		twitchClient!!.chat.joinChannel(getTwitchNameById("120275141"))

		twitchClient!!.eventManager.onEvent(ChannelMessageEvent::class.java) { event ->
			if (event.customRewardId.isPresent && event.customRewardId.get() == "38157e62-de35-4a21-8200-447b55d7577e") {
				logger.info { "Channel points redeemed: ${event.customRewardId.get()}" }
				logger.info { "Message: ${event.message}" }
				runBlocking {
					launch {
						// Send a message to the channel
						val channel = botRef!!.kordRef.getGuildOrNull(Snowflake(1130932838200512584))!!
							.getChannelOf<GuildMessageChannel>(Snowflake(1130954956892029060))
						val replay = ReplayCollection().get(event.message) ?: return@launch
						channel.createMessage {
							content = "Replay redeemed by: ${event.user.name}"
							this.addFile("replay.osr", ChannelProvider(null) { ByteReadChannel(replay.replayFile) })
 						}
						// remove the replay from the database
						ReplayCollection().delete(event.message)
					}
				}
			}
		}

		twitchClient!!.eventManager.onEvent(ChannelPointsCustomRewardRedemptionEvent::class.java) { event ->
			if (event.reward.id == "bd678e34-5a33-4d79-9cb0-fbb83f88a26b") {
				logger.info { "Channel points redeemed: ${event.id}" }
				logger.info { "Activating free Replays..." }
				runBlocking {
					launch {
						val channel = botRef!!.kordRef.getGuildOrNull(Snowflake(1130932838200512584))!!
							.getChannelOf<GuildMessageChannel>(Snowflake(1130954956892029060))
						ConfigCollection().update(ConfigCollection().get()!!.copy(replaySession = true))
						channel.createMessage {
							content = "Free Replays activated by: ${event.userName}"
						}
					}
				}
				logger.info { "Free Replays activated!" }
			}
		}
	}
}
